#ifndef _PAGE_H
#define _PAGE_H

// The fact that this is only 512 means that we could use unsigned shorts
// for a slot_id, and, in practice, an unsigned byte.  Oh well.

#define PAGESIZE 512

struct Page {
	char data[PAGESIZE];
};

#endif  // _PAGE_H
