#ifndef _GLOBAL_DEFS_H
#define _GLOBAL_DEFS_H

// ================================================================
// All the status codes

enum Status {
	OK,
	// random_rw
	NEGATIVE_PAGE_NUMBER,	// 1
	LSEEK_ERROR,		// 2
	READ_ERROR,		// 3
	WRITE_ERROR,		// 4
	NUM_CODES
};

#endif
